//modified by Jing Yang after 7/6/2001
#ifndef CSunburstDisplay_H
#define CSunburstDisplay_H

#include "CDisplay.H"
#include "CFlatDisplayInformation.H"
#include "CSunburstClusterTree.H"
#include "CSunburstCluster.H"

enum MOUSE_MODE {
    INACTIVE,
    LEFT_DISTORTION_HANDLE,
    RIGHT_DISTORTION_HANDLE,
    MIDDLE_DISTORTION,
};

class CSunburstDisplay: public CDisplay
{
public:
	ARR<double> radius;
	Vec2 center; 
	double circle_offset;

protected:
    CSunburstClusterTree *tree;
	CSunburstCluster* current_node;
    CSunburstCluster* prev_node;
	bool start_node;
	bool end_node;
	bool in_node;// if mouse is near the start of, the end of, or in the current_node, start>end>in

	MOUSE_MODE near_mouse_mode;
	MOUSE_MODE mouse_mode;

public:
	CSunburstDisplay();
	~CSunburstDisplay();
	
    //draw the canvas
    void Draw();
    virtual void InputMouseMotion(int x, int y);
	virtual void InputButtonPress(int button, int x, int y);
	virtual void InputButtonRelease(int button, int x, int y);

protected:
    virtual CSunburstClusterTree* GetTree();
	
    void TraceDrawClusters(CSunburstCluster *node);

	void TraceDrawClusterOutlines(CSunburstCluster *node);

    void TraceDrawClusterNames(CSunburstCluster *node);
	
	Vec2 MapData(double radius, double angle);
	
	//draw a single cluster
	void DrawSingleCluster(CSunburstCluster *c);
	
	//draw a single cluster outline
	void DrawSingleClusterOutline(CSunburstCluster *node);

    //draw a single cluster name
	virtual void DrawSingleClusterName(CSunburstCluster *node);
	
	//set parameters used for the drawing; to be rewritten by subclasses;
	void SetParameters();
	
	//free space used for drawing; to be rewritten by subclasses;
	void FreeSpace();
	
	void DrawCircles();

	CSunburstCluster* FindNode(double x, double y);
	
	CSunburstCluster* FindNode(double x, double y, bool &start, bool &end, bool &in);

	void GetLevelAndAngleAccordingXY(double x, double y, int &level, double &angle);

	virtual void SunburstCaption(int x, int y);

	bool GetAngleAccordingXY(double x, double y, double &angle);

};

#endif

